## R code for checking comparative statics of model fit with respect to parameters

## Settings
#datafile <- "../c/data/p20.txt"
datafile <- "../c/data/medD-full-const.sl3"
paramfile <- "xBest.txt" #"../c/x0.p20.2013-01-25"
probKeep <- 0.05
probKeep65 <- 0.5
nplanUse <- -500
outDir <- "figures/full/"
fdims <-1.5*c(5.03937, 3.77953) ## figure dimensions, =128mm x 96mm = dimensions of beamer slide
nsim <- 1 ## simulations per observation
nbin <- 75
require(RColorBrewer)
colors <- brewer.pal(8,"Dark2") ## colors for figures
colors <- colors[c(1:3,5,8)]
forceFig <- TRUE ## recreate figures even if they already exist?

if (!exists("result")) {
  ## run simulations using C++ library
  cflib <- "cf.so"
  if (is.loaded("cfsim")) dyn.unload(cflib)
  dyn.load(cflib)
  rate <- matrix(c(1,0.25,1,.07),nrow=1)
  dic <-matrix(c(275, 2510, 4050), nrow=1)
  cfname <- c("baseline","mu","sigma","lambda","p_omega","delta","d0","d.25","d.5","d.75","d1")
  args <- list(datafile,paramfile,nsim,rate,dic,compStatics=TRUE,probKeep,probKeep65,nplanUse)
  names(args) <- c("datafile","xfile","nsim","rate","dic","compStatics","probKeep","probKeep65","nplanUse")
  result <- .Call("cfsim",args)
} else {
  warning("Simulation result already exists, not re-running.")
}

## theme to mimic Liran's excel figures
fontFamily <- "Helvetica" ## change to Calibri if it's available
themeLE <- function(base_size=12, base_family=fontFamily) {
  theme_bw() %+replace%
  theme(axis.line = element_line(colour="grey"),
        panel.grid.minor = element_blank(),
        panel.background= element_blank(),
        panel.grid.major.x = element_blank(),
        panel.background = element_rect(fill="white",colour =NA),
        panel.border=element_blank(),
        text = element_text(family=base_family, size=base_size,
          face="plain", hjust=0.5, vjust=0.5, angle=0, lineheight=0.9,
          colour="black"),
        axis.text.x = element_text(angle=60),
        legend.position="bottom")
}
printDev <- function(filename, width=7, height=7, family=fontFamily) {
  cairo_pdf(filename,width=width,height=height,family=family)
  #pdf(filename,width,height)
}

################################################################################
getSimDF <- function(resList, cdat) {
  N <- length(cdat$joinweek)
  T <- length(resList[[1]][[1]]$cost)/N
  id <- 1:N
  idc <- as.vector(rep(1,T) %*% t(id))
  tc <- as.vector((1:T) %*% t(rep(1,N))) - 1
  df <- data.frame(id,joinweek=cdat$joinweek,age=cdat$age,rs=cdat$rs,weight=cdat$weigh)
  for (j in 1:length(resList)) {
    df[,sprintf("totcost.%d",j)] <- resList[[j]][[1]]$cumCost[tc==(T-1)]
    cat(j)
  }
  return(df)
}

df <- getSimDF(result$scf,result$Observed)

## weighted quantile, sd, and median
require(quantreg)
weighted.quantile <- function(x,probs=seq(0,1,0.25),w) {
  if (length(probs)>1) rq(x ~ 1, tau=probs,weights=w)$fitted.values[1,]
  else rq(x ~ 1, tau=probs,weights=w)$fitted.values[1]
}
weighted.median <- function(x,w) {
  weighted.quantile(x,0.5,w)
}
weighted.sd <- function(x,w) {
  xm <- weighted.mean(x,w)
  sqrt(sum(w*(x-xm)^2)/sum(w))
}
require(ggplot2)

filename <- paste(outDir,"bunchDelta.pdf",sep="")
if (!file.exists(filename) || forceFig) {  
  icl <- dic[1,2]
  xlim <- c(-1000,1000)
  bw <- (xlim[2]-xlim[1])/nbin
  fig <- ggplot(data=df,
                aes(x=totcost.7-icl,y=..density..,fill="Delta=0",weight=weight)) + themeLE() + 
    geom_histogram(alpha=0.3,colour=colors[1],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.8-icl, fill="Delta=0.25"),alpha=0.3,colour=colors[2],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.9-icl, fill="Delta=0.5"),alpha=0.3,colour=colors[3],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.10-icl, fill="Delta=0.75"),alpha=0.3,colour=colors[4],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.11-icl, fill="Delta=1.0"),alpha=0.3,colour=colors[5],position="identity",binwidth=bw) +
    scale_x_continuous(name="total spending - ICL",limits=xlim) +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())
  pdf(filename,width=fdims[1],height=fdims[2])
  print(fig)
  dev.off()
}

filename <- paste(outDir,"bunchDelta-ded.pdf",sep="")
if (!file.exists(filename) || forceFig) {  
  ded <- dic[1,1]
  xlim <- c(-ded+1,ded)
  bw <- (xlim[2]-xlim[1])/nbin
  fig <- ggplot(data=df,
                aes(x=totcost.7-ded,y=..density..,fill="Delta=0",weight=weight)) + themeLE() + 
    geom_histogram(alpha=0.3,position="identity",binwidth=bw,colour=colors[1]) +
    geom_histogram(aes(x=totcost.8-ded, fill="Delta=0.25"),alpha=0.3,colour=colors[2],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.9-ded, fill="Delta=0.5"),alpha=0.3,colour=colors[3],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.10-ded, fill="Delta=0.75"),alpha=0.3,colour=colors[4],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.11-ded, fill="Delta=1.0"),alpha=0.3,colour=colors[5],position="identity",binwidth=bw) +
    scale_x_continuous(name="total spending - deductible",limits=xlim) +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())
  pdf(filename,width=fdims[1],height=fdims[2])
  print(fig)
  dev.off()
}

######################################################################
filename <- paste(outDir,"bunchOmega.pdf",sep="")
if (!file.exists(filename) || forceFig) {  
  icl <- dic[1,2]
  xlim <- c(-1000,1000)
  bw <- (xlim[2]-xlim[1])/nbin
  fig <- ggplot(data=df,
                aes(x=totcost.12-icl,y=..density..,fill="p_omega=0",weight=weight)) + themeLE() + 
    geom_histogram(alpha=0.3,colour=colors[1],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.13-icl, fill="p_omega=0.25"),alpha=0.3,colour=colors[2],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.14-icl, fill="p_omega=0.5"),alpha=0.3,colour=colors[3],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.15-icl, fill="p_omega=0.75"),alpha=0.3,colour=colors[4],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.16-icl, fill="p_omega=1.0"),alpha=0.3,colour=colors[5],position="identity",binwidth=bw) +
    scale_x_continuous(name="total spending - ICL",limits=xlim) +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())
  pdf(filename,width=fdims[1],height=fdims[2])
  print(fig)
  dev.off()
}

filename <- paste(outDir,"bunchOmega-ded.pdf",sep="")
if (!file.exists(filename) || forceFig) {  
  ded <- dic[1,1]
  xlim <- c(-ded+1,ded)
  bw <- (xlim[2]-xlim[1])/nbin
  fig <- ggplot(data=df,
                aes(x=totcost.12-ded,y=..density..,fill="p_omega=0",weight=weight)) + themeLE() + 
    geom_histogram(alpha=0.3,position="identity",binwidth=bw,colour=colors[1]) +
    geom_histogram(aes(x=totcost.13-ded, fill="p_omega=0.25"),alpha=0.3,colour=colors[2],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.14-ded, fill="p_omega=0.5"),alpha=0.3,colour=colors[3],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.15-ded, fill="p_omega=0.75"),alpha=0.3,colour=colors[4],position="identity",binwidth=bw) +
    geom_histogram(aes(x=totcost.16-ded, fill="p_omega=1.0"),alpha=0.3,colour=colors[5],position="identity",binwidth=bw) +
    scale_x_continuous(name="total spending - deductible",limits=xlim) +
    scale_fill_manual(values=colors) +
    scale_colour_manual(values=colors) +
    theme(legend.position = "bottom", legend.title=element_blank())
  pdf(filename,width=fdims[1],height=fdims[2])
  print(fig)
  dev.off()
}
