## R code for reporting results of counterfactural simulations
library(RColorBrewer)
library(ggplot2)
library(data.table)
library(Rcpp)

paramfile <- "../estimation/baseline-pc3/smmEstCMA_2014-09-07_23.30.29.txt"
cfgFile <- "../estimation/baseline-pc3/baseline3.cfg"
outDir <- "figures/baseline-pc3/"

datafile <- "../estimation/data/medD-full-const.sl3"
nplanUse <- 500
probKeep <- 0.1
probKeep65 <- 0.1

fdims <-1.5*c(5.03937, 3.77953) ## figure dimensions, =128mm x 96mm = dimensions of beamer slide
nsim <- 1 ## simulations per observation
nbin <- 75
colors <- brewer.pal(8,"Dark2") ## colors for figures
colors <- colors[c(1:3,5,8)]
forceFig <- TRUE ## recreate figures even if they already exist?

## read configuration
cfg.txt <- readLines(cfgFile)
cfg.txt <- gsub("true","TRUE",cfg.txt)
cfg.txt <- gsub("false","FALSE",cfg.txt)
cfg <- new.env()
eval(parse(text=cfg.txt),envir=cfg)

NX <- cfg$NX + 1
raEst <- cfg$estRiskAversion
nmix <- cfg$NMIX
nlambda <- cfg$nLambda
singleLambdaRatio <- cfg$singleLambdaRatio
terminalV <- cfg$terminalV

## table of parameter estimates
parameters <- function(filename,out.file="x.tmp") {
  ## reads parameters from filename
  lines <- readLines(filename)
  if (grepl("x0",lines[1])) { ## working on smmEst***.txt file
    if (length(lines)!=3) {
      stop("expected 3 lines in paramfile")
    }
    if (grepl("x",lines[2])==FALSE) {
      stop("expected \"x\" in second line of paramfile")
    }
    x <- as.numeric(strsplit(lines[2]," +")[[1]][-1])
  } else {
    x <- scan(filename)
  }
  cat(format(x,digits=16),file=out.file)
  
  j <- 0
  mu <- cumsum(x[j + (1:nmix)])
  j <- j+nmix
  sig <-x[j + (1:nmix)]
  j <- j+nmix
  bmix <- t(matrix(c(rep(0,NX),x[j + 1:((nmix-1)*NX)]),nrow=NX))
  j <- j+(nmix-1)*NX
  pom <- x[j + (1:nmix)]
  j <- j+nmix
  lam <- matrix(NA,nrow=nmix,ncol=nlambda)
  lamRat <- rep(NA,nlambda)
  for(i in 1:nmix) {
    for(k in 1:nlambda) {
      if (singleLambdaRatio) {
        if (k==1) {
          lam[i,k] = x[j+1]
          j <- j+1;
        } else {
          if (i==1) {
            lamRat[k] = x[j+1]
            j <- j+1
          }
          lam[i,k] <- lamRat[k]*lam[i,k-1]
        }
      } else {
        if (k==1) {
          lam[i,k] = x[j+1]
          j <- j+1;
        } else {
          lam[i,k] = lam[i,k-1]*x[j+1]
          j <- j+1
        }
      }
    }
  }
  delta <- x[j+1]
  j <- j+1
  if (terminalV) {
    deltaOmega <- x[j+1]
    j <- j+1
    deltaTheta <- deltaOmega
  }   else {
    deltaOmega <- 0
    deltaTheta <- 0
  }
  if (raEst) {
    riskAversion <- exp(x[j+1])
  } else {
    riskAversion <- NA
  }
  transitionProb <- matrix(NA,nrow=nlambda,ncol=nlambda)
  for(i in 1:nlambda) {
    for(k in 1:nlambda) {
      if (i==k)
        transitionProb[k,i] = 1
      else {
        transitionProb[k,i] <- x[j+1]
        j <- j+1
      }
    }
    transitionProb[,i] <- transitionProb[,i]/sum(transitionProb[,i])
  }
  marginalProb <- lm.fit(x=rbind(transitionProb - diag(x=1,nrow=nlambda),rep(1,nlambda)),
                         y=c(rep(0,nlambda),1))$coefficients
  out <-     list( mu , sig , bmix , pom , lam , delta , riskAversion ,
                  transitionProb ,  marginalProb , deltaOmega, deltaTheta )
  names(out) <- c("mu","sig","bmix","pom","lam","delta","riskAversion",
                  "transitionProb","marginalProb", "deltaOmega", "deltaTheta")

  if (j != length(x)) {
    stop(cat("length(x) = ", length(x), " but only used ", j, " elements"))
  }
  return(out)
}


## I use this file interactively, so time consuming portions of code
## that I do not want to re-run everytime are wrapped in if {} else {}
## blocks
if (!exists("result")) {
  ## run simulations using C++ library
  cflib <- "cf.so"
  if (is.loaded("cfsim")) dyn.unload(cflib)
  dyn.load(cflib)
  srate <- c(1.0, 0.25, 1.0, 0.07)  ## std benefit
  drate <- c(0.01, 0.025, 0.03, 0.035, 0.05, 0.075, 0.1, 0.15, 0.25, 0.5, 0.75)
  rate <- rbind(matrix(srate,nrow=1),  ## std benefit
                matrix(1-drate,ncol=1) %*% matrix(srate,nrow=1)
                ## matrix(srate-0.01,nrow=1),  ## std benefit
                ## matrix(srate-0.02,nrow=1),  ## std benefit
                ## matrix(srate-0.03,nrow=1),  ## std benefit
                ## matrix(srate-0.04,nrow=1),  ## std benefit
                ## matrix(srate-0.05,nrow=1)  ## std benefit
                )
  dic <- t(matrix(rep(c(275, 2510, 4050),nrow(rate)) ## std benefit
                  , nrow=3))
  constTotCat <- FALSE
  if (constTotCat) {
    tcat <- dic[1,2] + (dic[1,3]-(dic[1,2]-dic[1,1]*rate[1,1])*rate[1,2] - dic[1,1]*rate[1,1])/rate[1,3]
    for(j in 2:nrow(dic)) {
      dic[j,3] <- (tcat - dic[j,2])*rate[j,3] + (dic[j,2]-dic[j,1]*rate[j,1])*rate[j,2] + dic[j,1]*rate[j,1]
    }
  }
  cfname <- c("standard benefit",
              sprintf("%.1f\\%% decrease",100*drate))
  tmp.file <- paste("x",as.numeric(Sys.time()),".tmp",sep="")
  pm <- parameters(paramfile,tmp.file)
  args <- list(datafile,tmp.file,nsim,rate,dic,compStatics=FALSE,probKeep,probKeep65,nplanUse,cfgFile)
  names(args) <- c("datafile","xfile","nsim","rate","dic","compStatics","probKeep","probKeep65",
                   "nplanUse","cfgFile")
  result <- .Call("cfsim",args)
  unlink(tmp.file)
} else {
  warning("Simulation result already exists, not re-running.")
}

if (!exists("simCost")) {
  ## put results into nice data structures
  id <- 1:length(result$Observed$plan)
  ## constant data
  conData <- data.frame(id,result$Observed[c("joinweek","plan","ded","icl","cat",
                                             "weight","age","rs")])
  conStd <- data.frame(id,result$cf[[1]][c("joinweek","plan","ded","icl","cat",
                                           "weight","age","rs")])
  nhData <- data.frame(id,result$NoHole[c("joinweek","plan","ded","icl","cat",
                                          "weight","age","rs")])
  ## observed cost data
  N <- length(id)
  T <- length(result$Observed$cost)/N
  idc <- as.vector(rep(1,T) %*% t(id))
  tc <- as.vector((1:T) %*% t(rep(1,N))) - 1
  jw <- as.vector(rep(1,T) %*% t(conData$joinweek))
  costData <- data.frame(idc,tc,result$Observed[c("cost","cumCost")],jw)
  names(costData)[1:2] <- c("id","t")
  costData <- costData[costData$t>=costData$jw,]
  costData <- costData[with(costData,order(id,t)),]
  ## simulated cost data
  nsim <- length(result$Simulated)
  simDF <- function(resultList,cdat) {
    jw <- as.vector(rep(1,T) %*% t(cdat$joinweek))
    df <- data.frame(idc,tc,resultList[[1]][c("cost","cumCost")],jw)
    df$sim <- 1
    if (nsim>1) {
      for(s in 2:nsim) {
        sim <- rep(s,length(idc))
        df <- rbind(df,
                    data.frame(idc,tc,resultList[[s]][c("cost","cumCost")],jw,sim))
      }
    }
    names(df) <- c(names(costData),"sim")
    df <- df[df$t>=df$jw,]
    df <- df[with(df,order(sim,id,t)),]
    return(df)   
  }

  scf <- list()
  simCost <- simDF(result$Simulated,conData)
  simNoHole <- simDF(result$SimNoHole,nhData)
  dcf <- list()
  for(j in 1:length(result$scf)) {
    dcf[[j]] <- simDF(result$scf[[j]],conStd)
  }
} else {
  warning("Simulation results already in data.frames, not re-creating.")
}

T <- max(costData$t)

totalSpend <- function(df,time=T) {
  subset(df,t==time)$cumCost
}

meanSpend <- lapply(dcf, function(x) { weighted.mean(totalSpend(x),conData$weight)} )

filename <- paste(outDir,"elasticities.tex",sep="")
cat("\\begin{table}[ht]\\begin{center}\\begin{tabular}{ccc}
 & Average Spending & Elasticity \\\\ \n",file=filename,append=FALSE)
cat(sprintf("Baseline & %.0f & --- \\\\ \n",meanSpend[[1]]),file=filename,append=TRUE)
#cat(sprintf("\\multicolumn{3}{l}{Reduced coinsurance rate by %.1f\\%%} \\\\ \n",
#            100*drate),file=filename,append=TRUE)
for(j in 2:length(meanSpend)) {
  cat(sprintf("%s & %.0f & %.3f \\\\ \n",cfname[j],meanSpend[[j]],
              (meanSpend[[j]]-meanSpend[[1]])/meanSpend[[1]]/(-drate[j-1]))
      ,file=filename,append=TRUE)
}
cat("\\end{tabular}\\end{center}\\end{table}\n",file=filename,append=TRUE)  


