## R code for reporting results of counterfactural simulations
##Rprof(filename="run.Rprof",memory.profiling=TRUE)
library(RColorBrewer)
library(quantreg)
library(ggplot2)
library(data.table)
library(Rcpp)

## Settings
paramfile <- "../estimation/baseline-pc3/smmEstCMA_2014-09-07_23.30.29.txt"
cfgFile <- "../estimation/baseline-pc3/baseline3.cfg"
outDir <- "figures/baseline-pc3/"

datafile <- "../estimation/data/medD-2010-const.sl3"
nplanUse <- 500
probKeep <- 0.1
probKeep65 <- 0.1
fdims <-1.5*c(5.03937, 3.77953) ## figure dimensions, =128mm x 96mm = dimensions of beamer slide
nsim <- 1 ## simulations per observation
nbin <- 75
colors <- brewer.pal(8,"Dark2") ## colors for figures
colors <- colors[c(1:3,5,8)]
forceFig <- TRUE ## recreate figures even if they already exist?

## read configuration
cfg.txt <- readLines(cfgFile)
cfg.txt <- gsub("true","TRUE",cfg.txt)
cfg.txt <- gsub("false","FALSE",cfg.txt)
cfg <- new.env()
eval(parse(text=cfg.txt),envir=cfg)

NX <- cfg$NX + 1
raEst <- cfg$estRiskAversion
nmix <- cfg$NMIX
nlambda <- cfg$nLambda
singleLambdaRatio <- cfg$singleLambdaRatio
terminalV <- cfg$terminalV

## theme to mimic Liran's excel figures
fontFamily <- "Helvetica" ## change to Calibri if it's available
themeLE <- function(base_size=12, base_family=fontFamily) {
  theme_bw() %+replace%
  theme(axis.line = element_line(colour="grey"),
        panel.grid.minor = element_blank(),
        panel.background= element_blank(),
        panel.grid.major.x = element_blank(),
        panel.background = element_rect(fill="white",colour =NA),
        panel.border=element_blank(),
        text = element_text(family=base_family, size=base_size,
          face="plain", hjust=0.5, vjust=0.5, angle=0, lineheight=0.9,
          colour="black"),
        axis.text.x = element_text(angle=60),
        legend.position="bottom")
}
printDev <- function(filename, width=7, height=7, family=fontFamily) {
  cairo_pdf(filename,width=width,height=height,family=family)
                                        #pdf(filename,width,height)
}

parameters <- function(filename,out.file="x.tmp") {
  ## reads parameters from filename
  lines <- readLines(filename)
  if (grepl("x0",lines[1])) { ## working on smmEst***.txt file
    if (length(lines)!=3) {
      stop("expected 3 lines in paramfile")
    }
    if (grepl("x",lines[2])==FALSE) {
      stop("expected \"x\" in second line of paramfile")
    }
    x <- as.numeric(strsplit(lines[2]," +")[[1]][-1])
  } else {
    x <- scan(filename)
  }
  cat(format(x,digits=16),file=out.file)
  
  j <- 0
  mu <- cumsum(x[j + (1:nmix)])
  j <- j+nmix
  sig <-x[j + (1:nmix)]
  j <- j+nmix
  bmix <- t(matrix(c(rep(0,NX),x[j + 1:((nmix-1)*NX)]),nrow=NX))
  j <- j+(nmix-1)*NX
  pom <- x[j + (1:nmix)]
  j <- j+nmix
  lam <- matrix(NA,nrow=nmix,ncol=nlambda)
  lamRat <- rep(NA,nlambda)
  for(i in 1:nmix) {
    for(k in 1:nlambda) {
      if (singleLambdaRatio) {
        if (k==1) {
          lam[i,k] = x[j+1]
          j <- j+1;
        } else {
          if (i==1) {
            lamRat[k] = x[j+1]
            j <- j+1
          }
          lam[i,k] <- lamRat[k]*lam[i,k-1]
        }
      } else {
        if (k==1) {
          lam[i,k] = x[j+1]
          j <- j+1;
        } else {
          lam[i,k] = lam[i,k-1]*x[j+1]
          j <- j+1
        }
      }
    }
  }
  delta <- x[j+1]
  j <- j+1
  if (terminalV) {
    deltaOmega <- x[j+1]
    j <- j+1
    deltaTheta <- deltaOmega
  }   else {
    deltaOmega <- 0
    deltaTheta <- 0
  }
  if (raEst) {
    riskAversion <- exp(x[j+1])
  } else {
    riskAversion <- NA
  }
  transitionProb <- matrix(NA,nrow=nlambda,ncol=nlambda)
  for(i in 1:nlambda) {
    for(k in 1:nlambda) {
      if (i==k)
        transitionProb[k,i] = 1
      else {
        transitionProb[k,i] <- x[j+1]
        j <- j+1
      }
    }
    transitionProb[,i] <- transitionProb[,i]/sum(transitionProb[,i])
  }
  marginalProb <- lm.fit(x=rbind(transitionProb - diag(x=1,nrow=nlambda),rep(1,nlambda)),
                         y=c(rep(0,nlambda),1))$coefficients
  out <-     list( mu , sig , bmix , pom , lam , delta , riskAversion ,
                  transitionProb ,  marginalProb , deltaOmega, deltaTheta )
  names(out) <- c("mu","sig","bmix","pom","lam","delta","riskAversion",
                  "transitionProb","marginalProb", "deltaOmega", "deltaTheta")

  if (j != length(x)) {
    stop(cat("length(x) = ", length(x), " but only used ", j, " elements"))
  }
  return(out)
}

if (!exists("result")) {
  ## run simulations using C++ library
  cflib <- "cf.so"
  if (is.loaded("cfsim")) dyn.unload(cflib)
  dyn.load(cflib)
  rate <- t(matrix(c(1.0, 0.25, 1.0, 0.07,  ## std benefit
                     1.0, 0.25, 0.25, 0.07, ## filled gap
                     1.0, 0.25, 1.0, 0.07,  ## no deductible
                                        #1.0, 0.25, 0.07, 0.07, ## generously filled gap
                     0.0, 0.0,  0.0, 0.0,   ## full insurance       
                     1.0, 1.0,  1.0, 1.0,   ## no insurance         
                     1.0, 1.0,  1.0, 0.07   ## only catastrophic    
                     ),nrow=4))
  dic <- t(matrix(c(275, 2510, 4050, ## std benefit
                    275, 2510, 4050, ## filled gap
                    0, 2510, 4050, ## no deductible
                                        #275, 2510, 4050, ## generously filled gap
                    275, 2510, 4050, ## full insurance       
                    275, 2510, 4050, ## no insurance         
                    275, 2510, 4050  ## only catastrophic    
                    ), nrow=3))
  cfname <- c("standard benefit",
              "filled gap",
              "no deductible",
              ##"more filled gap",
              "full insurance",
              "no insurance",
              "only catastrophic coverage")
  tmp.file <- paste("x",as.numeric(Sys.time()),".tmp",sep="")
  pm <- parameters(paramfile,tmp.file)
  args <- list(datafile,tmp.file,nsim,rate,dic,compStatics=FALSE,probKeep,probKeep65,nplanUse,cfgFile)
  names(args) <- c("datafile","xfile","nsim","rate","dic","compStatics","probKeep","probKeep65","nplanUse",
                   "cfgFile")
  result <- .Call("validationFit",args)
  unlink(tmp.file)
} else {
  warning("Simulation result already exists, not re-running.")
}

if (!exists("simCost")) {
  ## put results into nice data structures
  id <- 1:length(result$Observed$plan)
  ## constant data
  conData <- data.frame(id,result$Observed[c("joinweek","plan","ded","icl","cat",
                                             "weight","age","rs","nogap")])
  ## observed cost data
  N <- length(id)
  T <- length(result$Observed$cost)/N
  idc <- as.vector(rep(1,T) %*% t(id))
  tc <- as.vector((1:T) %*% t(rep(1,N))) - 1
  jw <- as.vector(rep(1,T) %*% t(conData$joinweek))
  costData <- data.frame(idc,tc,result$Observed[c("cost","cumCost")],jw)
  names(costData)[1:2] <- c("id","t")
  costData <- costData[costData$t>=costData$jw,]
  costData <- costData[with(costData,order(id,t)),]

  id <- 1:length(result$DiffObs$plan)
  conDiffObs <- data.frame(id,result$DiffObs[c("joinweek","plan","ded","icl","cat",
                                               "weight","age","rs","nogap")])
  N <- length(id)
  T <- length(result$DiffObs$cost)/N
  idc <- as.vector(rep(1,T) %*% t(id))
  tc <- as.vector((1:T) %*% t(rep(1,N))) - 1
  jw <- as.vector(rep(1,T) %*% t(conDiffObs$joinweek))
  costDiffObs <- data.frame(idc,tc,result$DiffObs[c("cost","cumCost")],jw)
  names(costDiffObs)[1:2] <- c("id","t")
  costDiffObs <- costDiffObs[costDiffObs$t>=costDiffObs$jw,]
  costDiffObs <- costDiffObs[with(costDiffObs,order(id,t)),]

  id <- 1:length(result$DiffPlans$plan)
  conDiffPlans <- data.frame(id,result$DiffPlans[c("joinweek","plan","ded","icl","cat",
                                                    "weight","age","rs","nogap")])
  N <- length(id)
  T <- length(result$DiffPlans$cost)/N
  idc <- as.vector(rep(1,T) %*% t(id))
  tc <- as.vector((1:T) %*% t(rep(1,N))) - 1
  jw <- as.vector(rep(1,T) %*% t(conDiffPlans$joinweek))
  costDiffPlans <- data.frame(idc,tc,result$DiffPlans[c("cost","cumCost")],jw)
  names(costDiffPlans)[1:2] <- c("id","t")
  costDiffPlans <- costDiffPlans[costDiffPlans$t>=costDiffPlans$jw,]
  costDiffPlans <- costDiffPlans[with(costDiffPlans,order(id,t)),]
  
  id <- 1:length(result$Obs2010$plan)
  conObs2010 <- data.frame(id,result$Obs2010[c("joinweek","plan","ded","icl","cat",
                                                    "weight","age","rs","nogap")])
  N <- length(id)
  T <- length(result$Obs2010$cost)/N
  idc <- as.vector(rep(1,T) %*% t(id))
  tc <- as.vector((1:T) %*% t(rep(1,N))) - 1
  jw <- as.vector(rep(1,T) %*% t(conObs2010$joinweek))
  costObs2010 <- data.frame(idc,tc,result$Obs2010[c("cost","cumCost")],jw)
  names(costObs2010)[1:2] <- c("id","t")
  costObs2010 <- costObs2010[costObs2010$t>=costObs2010$jw,]
  costObs2010 <- costObs2010[with(costObs2010,order(id,t)),]

  ## simulated cost data
  simDF <- function(resultList,cdat) {
    N <- length(cdat$id)
    T <- length(resultList[[1]]$cost)/N
    nsim <- length(resultList)
    idc <- as.vector(rep(1,T) %*% t(cdat$id))
    tc <- as.vector((1:T) %*% t(rep(1,N))) - 1
    jw <- as.vector(rep(1,T) %*% t(cdat$joinweek))

    df <- data.frame(idc,tc,resultList[[1]][c("cost","cumCost")],jw)
    df$sim <- 1
    if (nsim>1) {
      for(s in 2:nsim) {
        sim <- rep(s,length(idc))
        df <- rbind(df,
                    data.frame(idc,tc,resultList[[s]][c("cost","cumCost")],jw,sim))
      }
    }
    names(df) <- c(names(costData),"sim")
    df <- df[df$t>=df$jw,]
    df <- df[with(df,order(sim,id,t)),]
    return(df)   
  }
  simCost <- simDF(result$Simulated,conData)
  simDiffObs <- simDF(result$SimDiffObs,conDiffObs)
  simDiffPlans <- simDF(result$SimDiffPlans,conDiffPlans)
  sim2010 <- simDF(result$Sim2010,conObs2010)
} else {
  warning("Simulation results already in data.frames, not re-creating.")
}
T <- max(costData$t)

fitFigures <- function(costData, simCost,conData, outDir) {
  figData <- list()
  ## create output directory if needed
  if (!file.exists(outDir)) {
    dir.create(outDir,recursive=TRUE)
  }

  ## histogram of total spending
  filename <- paste(outDir,"fit.pdf",sep="")
  if (!file.exists(filename) || forceFig) {
    maxc <- 6000 
    fd <- rbind(data.frame(x=subset(costData,t==T)$cumCost,w=conData$weight,group="observed"),
                data.frame(x=subset(simCost,t==T)$cumCost,w=conData$weight,group="estimated"))
    tab <- data.table(fd)[,list(p0=mean(x==0,na.rm=TRUE),
                                mean=mean(x,na.rm=TRUE),
                                sd=sd(x,na.rm=TRUE),
                                median=median(x,na.rm=TRUE)),by=group]
    fd$x[fd$x>maxc] <- maxc
    fitPlot <- ggplot(data=fd, aes(x=x,y=2*..count../sum(..count..),fill=group,weight=w)) + themeLE() + 
      stat_bin(breaks=seq(0.0,maxc+1,by=100),alpha=0.3,position="dodge") + 
                                        #geom_histogram(alpha=0.3,position="dodge",binwidth=maxc/nbin) +    
        scale_x_continuous(name="total spending") +
          scale_y_continuous(name="portion") + 
            scale_fill_manual(values=colors) + 
              theme(legend.position = "bottom",legend.title=element_blank()) +
                annotate("text",x=500,y=.06,
                         label=sprintf(" observed: P(0)=%.3f median=%.2f, mean=%.2f, and sd=%.2f",
                           tab$p0[1],tab$median[1],tab$mean[1],tab$sd[1]), size=4, hjust=0,color=colors[1]) + 
                             annotate("text",x=500,y=.05,
                                      label=sprintf("estimated: P(0)=%.3f median=%.2f, mean=%.2f, and sd=%.2f",
                                        tab$p0[2],tab$median[2],tab$mean[2],tab$sd[2]), size=4, hjust=0,color=colors[2]) 
    printDev(filename,width=fdims[1],height=fdims[2])
    print(fitPlot)
    dev.off()
    ## save plot data to create xls file for Liran :(
    figData <- c(figData, list(ggplot_build(fitPlot)$data[[1]]))
    names(figData)[length(figData)] <- "Fit.Histogram"
  } else {
    warning(paste(filename,"exists, not recreating"))
  }
  
  ## total spending near icl
  filename <- paste(outDir,"bunchFit.pdf",sep="")
  if (!file.exists(filename) || forceFig) {  
    fd <- rbind(data.frame(x=(subset(costData,t==T)$cumCost-conData$icl),w=conData$weight,
                           group="observed"),
                data.frame(x=(subset(simCost,t==T)$cumCost-conData$icl),w=conData$weight,
                           group="estimated"))
    xlim <- c(-1000,1000)
    xld <- xlim
    fd <- subset(fd,x>=xlim[1] & x<=xlim[2])
    bw <- 20
    fitPlot <- ggplot(data=fd, aes(x=x,y=..density..,fill=group, weight=w)) + themeLE() + 
      geom_histogram(alpha=0.3,position="dodge",binwidth=bw) +
        scale_x_continuous(name="total spending - ICL") +
          scale_fill_manual(values=colors) +
            scale_colour_manual(values=colors) +
              theme(legend.position = "bottom", legend.title=element_blank())  
    printDev(filename,width=fdims[1],height=fdims[2])
    print(fitPlot)
    dev.off()
    ## save plot data to create xls file for Liran :(
    figData <- c(figData, ggplot_build(fitPlot)$data[1:2])  
    names(figData)[(length(figData)-1):length(figData)] <-
      c("Fit.Histogram.NearICL","Fit.Density.NearICL")

    filename <- paste(outDir,"bunchFit-estWindow.pdf",sep="")
    fd <- rbind(data.frame(x=(subset(costData,t==T)$cumCost-conData$icl),
                           group="observed",w=conData$weight),
                data.frame(x=(subset(simCost,t==T)$cumCost-conData$icl),
                           group="estimated",w=conData$weight))
    xlim <- c(-500,500)
    xld <- xlim
    fd <- subset(fd,x>=xlim[1] & x<=xlim[2])
    bw <- (xlim[2]-xlim[1])/nbin
    fitPlot <- ggplot(data=fd, aes(x=x,y=..density..,fill=group,weight=w)) + themeLE() + 
      geom_histogram(alpha=0.3,position="dodge",binwidth=bw) +
        scale_x_continuous(name="total spending - ICL") +
          scale_fill_manual(values=colors) +
            scale_colour_manual(values=colors) +
              theme(legend.position = "bottom", legend.title=element_blank())  
    printDev(filename,width=fdims[1],height=fdims[2])
    print(fitPlot)
    dev.off()
    ## save plot data to create xls file for Liran :(
    figData <- c(figData, ggplot_build(fitPlot)$data[1:2])  
    names(figData)[(length(figData)-1):length(figData)] <-
      c("Fit.Histogram.NearICL2","Fit.Density.NearICL2")
  } else {
    warning(paste(filename,"exists, not recreating"))
  }

  ## Timing moments
  month.names <- c("January","February","March","April",
                   "May","June","July","August","September",
                   "October","November","December")
  filename <- paste(outDir,"timing",month.names[12],".pdf",sep="")
  if (!file.exists(filename) || forceFig) { 
    fd <- rbind(data.frame(x=(subset(costData,t==T)$cumCost-conData$icl),
                           group="observed",w=conData$weight),
                data.frame(x=(subset(simCost,t==T)$cumCost-conData$icl),
                           group="estimated",w=conData$weight))
    xlim <- c(-500,500)
    bins <- seq(-500,500,length.out=21)
    fd$bin <- NA
    for(i in 1:(length(bins)-1)) {
      fd$bin[fd$x>=bins[i] & fd$x<bins[i+1]] <- i
    }
    for (m in 6:12) {
      t <- T-(13-m)*4
      fd$y <- c((rowsum(costData$cost*(costData$t>=t),group=costData$id)>0),
                (rowsum(simCost$cost*(simCost$t>=t),group=costData$id)>0))
      y.obs <- rep(NA,length(bins)-1)
      y.sim <- y.obs
      for(i in 1:(length(bins)-1)) {
        y.obs[i] <- sum(fd$y[fd$group=="observed" & fd$bin==i],na.rm=TRUE) /
          sum(fd$bin==i & fd$group=="observed",na.rm=TRUE)
        y.sim[i] <- sum(fd$y[fd$group=="estimated" & fd$bin==i],na.rm=TRUE) /
          sum(fd$bin==i & fd$group=="estimated",na.rm=TRUE)
      }
      gd <- data.frame(x=(bins[2:length(bins)]+bins[1:(length(bins))-1])/2,
                       observed=y.obs,estimated=y.sim)
      filename <- paste(outDir,"timing",month.names[m],".pdf",sep="")
      fitPlot <- ggplot(data=gd, aes(x=x, y=observed,
                          color="observed", linetype="observed")) +
                            themeLE() + 
                              geom_line() + 
                                geom_line(aes(y=estimated,color="estimated",linetype="estimated")) + 
                                  scale_x_continuous(name="total spending - ICL") +
                                    scale_y_continuous(name=paste("P(claim in ",
                                                         month.names[m],
                                                         " or later | total spending)",sep="")) + 
                                                           scale_fill_manual(values=colors) +
                                                             scale_colour_manual(values=colors) +
                                                               theme(legend.position = "bottom", legend.title=element_blank())  
      printDev(filename,width=fdims[1],height=fdims[2])
      print(fitPlot)
      dev.off()
    }
    p.obs <- matrix(NA,nrow=13,ncol=2)
    p.sim <- matrix(NA,nrow=13,ncol=2)
    fd$bin <- NA
    fd$bin[fd$x>=-800 & fd$x<=-500] <- 1
    fd$bin[fd$x>=-150 & fd$x<=150] <- 2
    fd$bin[is.na(fd$bin)] <- 0
    for (m in 1:13) {
      m.ind <- costData$t==(m-1)*4 | costData$t==(m-1)*4+1 |
        costData$t==(m-1)*4+2 | costData$t==(m-1)*4+3
      y <- rowsum(costData$cost*m.ind,group=costData$id)>0
      for(i in 1:2) {
        p.obs[m,i] <- mean(y[fd$bin[fd$group=="observed"]==i])
      }
      m.ind <- simCost$t==(m-1)*4 | simCost$t==(m-1)*4+1 |
        simCost$t==(m-1)*4+2 | simCost$t==(m-1)*4+3
      y <- rowsum(simCost$cost*m.ind,group=simCost$id)>0
      for(i in 1:2) {
        p.sim[m,i] <- mean(y[fd$bin[fd$group=="estimated"]==i])
      }
    }
    gd <- data.frame(month=1:13,obs.pre=p.obs[,1],obs.kink=p.obs[,2],
                     sim.pre=p.sim[,1], sim.kink=p.sim[,2])
    fitPlot <- ggplot(data=gd, aes(x=month, y=obs.pre,
                        color="pre")) + 
                          themeLE() + 
                            geom_line(linetype=1) + 
                              geom_line(aes(y=obs.kink,color="kink"),linetype=1) +
                                geom_line(aes(y=sim.pre,color="pre"),linetype=2) +
                                  geom_line(aes(y=sim.kink,color="kink"),linetype=2) +
                                    scale_x_continuous(name="month") +
                                      scale_y_continuous(name="P(claim in month|total spending)") +
                                        scale_fill_manual(values=colors) +
                                          scale_colour_manual(values=colors) +
                                            theme(legend.position = "bottom", legend.title=element_blank())
    filename <- paste(outDir,"timingAll.pdf",sep="")
    printDev(filename,width=fdims[1],height=fdims[2])
    print(fitPlot)
    dev.off()
    figData <- c(figData, list(gd))
    names(figData)[length(figData)] <- "timingAll"
  } else {
    warning(paste(filename,"exists, not recreating"))
  }

  ## Autocorrelation of claims
  ## filename <- paste(outDir,"autocorr.pdf",sep="")
  ## if (FALSE) { #(!file.exists(filename) || forceFig) {
  ##   lags <- 20
  ##   lag <- 0:lags
  ##   cost.obs <- matrix(NA,ncol=(T+1),nrow=N)
  ##   cost.sim <- cost.obs
  ##   cost.obs[costData$id + costData$t*N] <- costData$cost
  ##   cost.sim[simCost$id + simCost$t*N] <- simCost$cost
  ##   p.obs <- rep(NA,lags+1)
  ##   p.sim <- p.obs
  ##   for(l in lag) {
  ##     p.obs[l+1] <- mean(cost.obs[,1:(T+1-l)]>0 & cost.obs[,(1+l):(T+1)]>0,na.rm=TRUE)
  ##     p.sim[l+1] <- mean(cost.sim[,1:(T+1-l)]>0 & cost.sim[,(1+l):(T+1)]>0,na.rm=TRUE)
  ##   }
  ##   gd <- data.frame(lag=lag,observed=p.obs,estimated=p.sim)
  ##   fitPlot <- ggplot(data=gd, aes(x=lag,y=observed,color="observed",linetype="observed")) + themeLE() +
  ##     geom_line() + 
  ##       geom_line(aes(y=estimated,color="estimated",linetype="estimated")) + 
  ##         scale_x_continuous(name="lag") +
  ##           scale_y_continuous(name=paste("P(claim t & t-lag)",sep="")) +
  ##             scale_fill_manual(values=colors) +
  ##               scale_colour_manual(values=colors) +
  ##                 theme(legend.position = "bottom", legend.title=element_blank())  
  ##   printDev(filename,width=fdims[1],height=fdims[2])
  ##   print(fitPlot)
  ##   dev.off()

  ##   lags <- 20
  ##   lag <- 1:lags
  ##   N <- max(costData$id)
  ##   cost.obs <- matrix(NA,ncol=(T+1),nrow=N)
  ##   cost.sim <- cost.obs
  ##   cost.obs[costData$id + costData$t*N] <- costData$cost
  ##   cost.sim[simCost$id + simCost$t*N] <- simCost$cost
  ##   p.obs <- rep(NA,lags)
  ##   p.sim <- p.obs
  ##   fe.obs <- p.obs
  ##   fe.sim <- p.sim
    
  ##   e.obs <- cost.obs - rowMeans(cost.obs,na.rm=TRUE)
  ##   e.sim <- cost.sim - rowMeans(cost.sim,na.rm=TRUE)

  ##   for(l in lag) {
  ##     c1 <- as.vector
  ##     p.obs[l] <- cor(as.vector(cost.obs[,1:(T+1-l)]),as.vector(cost.obs[,(1+l):(T+1)]),
  ##                     use="na.or.complete")
  ##     p.sim[l] <-  cor(as.vector(cost.sim[,1:(T+1-l)]),as.vector(cost.sim[,(1+l):(T+1)]),
  ##                      use="na.or.complete")

  ##     fe.obs[l] <- cor(as.vector(e.obs[,1:(T+1-l)]),as.vector(e.obs[,(1+l):(T+1)]),
  ##                      use="na.or.complete")
  ##     fe.sim[l] <-  cor(as.vector(e.sim[,1:(T+1-l)]),as.vector(e.sim[,(1+l):(T+1)]),
  ##                       use="na.or.complete")
      
  ##   }
  ##   gd <- data.frame(lag=lag,observed=p.obs,estimated=p.sim)
  ##   fitPlot <- ggplot(data=gd, aes(x=lag,y=observed,color="observed",linetype="observed")) + themeLE() +
  ##     geom_line() + 
  ##       geom_line(aes(y=estimated,color="estimated",linetype="estimated")) + 
  ##         scale_x_continuous(name="lag") +
  ##           scale_y_continuous(name=paste("Correlation spending at time t and t-lag",sep="")) +
  ##             scale_fill_manual(values=colors) +
  ##               scale_colour_manual(values=colors) +
  ##                 theme(legend.position = "bottom", legend.title=element_blank())
  ##   filename <- paste(outDir,"autocorr-spend.pdf",sep="")
  ##   printDev(filename,width=fdims[1],height=fdims[2])
  ##   print(fitPlot)
  ##   dev.off()

  ##   gd <- data.frame(lag=lag,observed=fe.obs,estimated=fe.sim)
  ##   fitPlot <- ggplot(data=gd, aes(x=lag,y=observed,color="observed",linetype="observed")) + themeLE() +
  ##     geom_line() + 
  ##       geom_line(aes(y=estimated,color="estimated",linetype="estimated")) + 
  ##         scale_x_continuous(name="lag") +
  ##           scale_y_continuous(name=paste("Correlation spending at time t and t-lag",sep="")) +
  ##             scale_fill_manual(values=colors) +
  ##               scale_colour_manual(values=colors) +
  ##                 theme(legend.position = "bottom", legend.title=element_blank())
  ##   filename <- paste(outDir,"autocorr-spend-fe.pdf",sep="")
  ##   printDev(filename,width=fdims[1],height=fdims[2])
  ##   print(fitPlot)
  ##   dev.off()

  ## } else {
  ##   warning(paste(filename,"exists, not recreating"))
  ## }
  
  return(figData)
}

#fit.inSample <- fitFigures(costData,simCost,conData,outDir)
fit.diffObs <- fitFigures(costDiffObs,simDiffObs,conDiffObs,paste(outDir,"diffObs/",sep="/"))
fit.diffPlans <- fitFigures(costDiffPlans,simDiffPlans,conDiffPlans,paste(outDir,"diffPlans/",sep="/"))
fit.2010 <- fitFigures(costObs2010,sim2010,conObs2010,paste(outDir,"2010/",sep="/"))

saveXLS <- function(figData,filename) {
  require(WriteXLS)
  fig7a <- data.frame(x=subset(figData$Fit.Histogram,group==1)$xmin,
                      Observed=subset(figData$Fit.Histogram,group==1)$ymax,
                      Predicted=subset(figData$Fit.Histogram,group==2)$ymax)
  fig7b <- data.frame(x=subset(figData$Fit.Histogram.NearICL,group==1)$xmin,
                      Observed=subset(figData$Fit.Histogram.NearICL,group==1)$ymax,
                      Predicted=subset(figData$Fit.Histogram.NearICL,group==2)$ymax)
  fig8 <- figData$timingAll
  WriteXLS(c("fig7a","fig7b","fig8"),
         ExcelFileName=filename,
         SheetNames=c("Figure 7a Data", "Figure 7b Data",
           "Figure 8 Data"))
}

saveXLS(fit.2010, paste(outDir,"2010/fitFig.xlsx",sep="/"))
saveXLS(fit.diffObs, paste(outDir,"diffObs/fitFig.xlsx",sep="/"))
saveXLS(fit.diffPlans, paste(outDir,"diffPlans/fitFig.xlsx",sep="/"))

## create slides
tokens <- strsplit(outDir,"/",1)
filename <- paste(tokens[[1]][length(tokens[[1]])],"OOS.pdf",sep="")
system(sprintf("cd ../../tex/slides && make %s",filename))

