#
rm(list=ls())
library(foreign)
library(xtable)
dat =read.dta("/disk/agedisk5/medicare.work/afinkels-DUA22559/schrimpf/Data/extract/data_extract.dta")
figureDirectory <- "figures"
tableDirectory <- "tables"

# sort by id, claim date
dat <- dat[order(dat$bene_id,dat$srvc_dt),]

# generate cumulative payment and cost
dat$cumPay <- ave(dat$ptpayamt,dat$bene_id, dat$yr, FUN=cumsum)
dat$cumCost <- ave(dat$totalcst,dat$bene_id,dat$yr,FUN=cumsum)

# generate plan segment indicator
dat$segment <- as.factor(1*(dat$cumCost<dat$ded_amt) +
                         2*(dat$cumCost>=dat$ded_amt & dat$cumCost<dat$icl_amt) +
                         3*(dat$cumCost>=dat$icl_amt & dat$cumPay<dat$oopt_amt) +
                         4*(dat$cumPay>=dat$oopt_amt))

# relative payment
dat$relPay <- dat$ptpayamt/dat$totalcst
dat$bnftphas <- factor(dat$bnftphas)
# look at one plan at a time
for (y in sort(unique(dat$yr))) {
  for (p in unique(dat$plan)) {
    subset <- (dat$yr==y & dat$plan==p &
               (dat$bnftphas=="CC" | dat$bnftphas=="DD" |
                dat$bnftphas=="PP" | dat$bnftphas=="II"))
  
    oopModel <- lm(cumPay ~ 0 + bnftphas + bnftphas:cumCost,data=dat, subset=subset)
    print(xtable(oopModel),file=sprintf("%s/oop%d_%s.tex",tableDirectory,y,p))
    relModel <- lm(relPay ~ 0+ bnftphas,data=dat, subset=subset)
    print(xtable(relModel),file=sprintf("%s/rel%d_%s.tex",tableDirectory,y,p))

    # plot of fit in levels
    pdf(file=sprintf("%s/oop%d_%s.pdf",figureDirectory,y,p))   
    plot(dat$cumCost[subset],dat$cumPay[subset],
         xlab="Cumulative spending", ylab="Cumulative patient payment")
    points(dat$cumCost[subset],
           predict(oopModel,newdata=dat[subset,]),
           pch=17, col="red")    
    dev.off()

    # plot of relative fit
    pdf(file=sprintf("%s/rel%d_%s.pdf",figureDirectory,y,p))   
    plot(dat$cumCost[subset],dat$relPay[subset],
         xlab="Cumulative spending", ylab="Relative patient payment")
    points(dat$cumCost[subset],
           predict(relModel,newdata=dat[subset,]),
           pch=17, col="red")    
    dev.off()

    # plot of relative fit in level-space
    payhat <- predict(relModel,newdata=dat[subset,])*dat$totalcst[subset]
    cphat <- ave(payhat,dat$bene_id[subset],dat$yr[subset],FUN=cumsum)
    pdf(file=sprintf("%s/rel-lev%d_%s.pdf",figureDirectory,y,p))   
    plot(dat$cumCost[subset],dat$cumPay[subset],
         xlab="Cumulative spending", ylab="Cumulative patient payment")
    points(dat$cumCost[subset],
           cphat,
           pch=17, col="red")    
    dev.off()
    
  }
}


## number of observations per person and year
newid <- c(TRUE,dat$bene_id[2:length(dat$bene_id)]!=
           dat$bene_id[1:(length(dat$bene_id)-1)])
newday <- c(TRUE,dat$bene_id[2:length(dat$bene_id)]!=
           dat$bene_id[1:(length(dat$bene_id)-1)] |
           dat$srvc_dt[2:length(dat$bene_id)]!=
           dat$srvc_dt[1:(length(dat$bene_id)-1)])
dat$nClaims <- ave(dat$ptpayamt,dat$bene_id,dat$srvc_dt,FUN=length)
dat$nObs <- ave(as.numeric(dat$srvc_dt),dat$bene_id,dat$yr,FUN=length)
dat$nDates <- ave(as.numeric(dat$srvc_dt),dat$bene_id,dat$yr,
                  FUN=function(x) { length(unique(x)) } )
dat$srvc_wk <- as.integer( (as.numeric(dat$srvc_dt) -
                            as.numeric(as.Date("2007-01-01")))/7)
ewwk <-  c(TRUE,dat$bene_id[2:length(dat$bene_id)]!=
            dat$bene_id[1:(length(dat$bene_id)-1)] |
            dat$srvc_wk[2:length(dat$bene_id)]!=
            dat$srvc_wk[1:(length(dat$bene_id)-1)])
dat$nWk <- ave(dat$srvc_wk,dat$bene_id,dat$yr,
               FUN=function(x) { length(unique(x)) } )
dat$nClaimWk <- ave(dat$ptpayamt,dat$bene_id,dat$srvc_wk,FUN=length)

dat$srvc_month <- (cut(dat$srvc_dt,"months",labels=FALSE)-1)%%12 + 1
dat$monthsin <- dat$srvc_month - dat$partd_joinmonth

# compute probability of claim by week and join month
dat$week <- as.integer( (as.numeric(dat$srvc_dt) -
                            as.numeric(as.Date(sprintf("%4d-01-01",dat$yr)))/7)                      
dat$joinweek <- as.integer((as.numeric(as.Date(sprintf("%4d-%2d-01",dat$yr,dat$partd_joinmonth)))
                            - as.numeric(as.Date(sprintf("%4d-01-01",dat$yr))) ) / 7) 

Pclaim <- function(t,sub) {
  sapply(t,         
         function(x) {
           length(unique(subset(dat$bene_id,dat$week==x & sub & dat$joinweek<=x) )) /
             length(unique(subset(dat$bene_id,sub & dat$joinweek<=x) )) } )      
}
PclaimJoinMonth <- sapply(1:12,
                          function(m) {
                            Pclaim(0:52,dat$partd_joinmonth==m)
                          })
PclaimJoinMonth[is.infinite(PclaimJoinMonth)] <- NA
PclaimAll <- data.frame(cbind(0:52,
                              Pclaim(0:52, !is.na(dat$week)),
                              PclaimJoinMonth))
names(PclaimAll) <- c("Week","All",sprintf("JoinMonth%d",1:12))
PclaimAll <- reshape(PclaimAll,direction="long",
                     varying=sprintf("JoinMonth%d",1:12),
                     sep="",idvar="Week")
names(PclaimAll)[names(PclaimAll)=="JoinMonth"] <- "Pclaim"
names(PclaimAll)[names(PclaimAll)=="time"] <- "JoinMonth"
PclaimAll$diff <- PclaimAll$Pclaim-PclaimAll$All
PclaimAll$JoinMonth <- as.factor(PclaimAll$JoinMonth)

library(ggplot2)
pp <- ggplot(data=PclaimAll,aes(x=Week,y=Pclaim,group=JoinMonth,
               color=JoinMonth)) +
  geom_line() + scale_color_hue() +
  geom_line(aes(y=All,color="All"),color="black",size=2)
##scale_color_brewer(type="qual") + ##scale_color_gradient2(mid="grey") + 
pdf(sprintf("%s/Pclaim.pdf",figureDirectory))
pp
dev.off()
